@@ -1,5 +1,5 @@
 from .numerical_expression import construct_expression_tree, calculate, evaluate_expression, NumericalExpressionTree
-from .observation import ActionCall, Observation, ObservedComponent
+from .observation import ActionCall, Observation, ObservedComponent, MultiAgentObservation, MultiAgentComponent
 from .pddl_action import NumericPrecondition, Action, NumericEffect
 from .pddl_domain import Domain
 from .pddl_function import PDDLFunction
