@@ -2,9 +2,10 @@
 import logging
 import re
 from pathlib import Path
-from typing import List, Tuple
+from typing import List, Tuple, Set
 
-from pddl_plus_parser.models import Domain, ActionCall, Operator, JointActionCall, NOP_ACTION
+from pddl_plus_parser.models import Domain, ActionCall, Operator, JointActionCall, NOP_ACTION, Problem, State
+from pddl_plus_parser.multi_agent.common import create_initial_state, apply_actions
 
 PLAN_COMPONENT_REGEX = r"\d+ : \(([\w+\s?-]+)\)"
 
@@ -41,6 +42,36 @@ class PlanConverter:
 
         return plan_seq
 
+    @staticmethod
+    def _validate_two_sided_relation(
+            positive_preconditions: Set[str], negative_preconditions: Set[str], add_effects: Set[str],
+            delete_effects: Set[str], next_action_pos_preconditions: Set[str],
+            next_action_neg_preconditions: Set[str], next_action_add_effects: List[str],
+            next_action_del_effects: List[str]) -> bool:
+        """
+
+        :param positive_preconditions:
+        :param negative_preconditions:
+        :param add_effects:
+        :param delete_effects:
+        :param next_action_pos_preconditions:
+        :param next_action_neg_preconditions:
+        :param next_action_add_effects:
+        :param next_action_del_effects:
+        :return:
+        """
+        if len(negative_preconditions.intersection(next_action_pos_preconditions)) > 0 or \
+                len(positive_preconditions.intersection(next_action_neg_preconditions)) > 0 or \
+                len(add_effects.intersection(next_action_del_effects)) > 0 or \
+                len(delete_effects.intersection(next_action_add_effects)) > 0 or \
+                len(positive_preconditions.intersection(next_action_del_effects)) > 0 or \
+                len(negative_preconditions.intersection(next_action_add_effects)) > 0 or \
+                len(next_action_pos_preconditions.intersection(delete_effects)) > 0 or \
+                len(next_action_neg_preconditions.intersection(add_effects)) > 0:
+            return False
+
+        return True
+
     def _validate_well_defined_action_literals(self, combined_actions: List[ActionCall], next_action: Operator) -> bool:
         """Validates whether the actions' grounded literals are well-defined.
 
@@ -54,6 +85,7 @@ class PlanConverter:
         :param next_action: the new action to consider to add to the joint action.
         :return: whether the grounded literals are well-defined.
         """
+        self.logger.debug("Validating that the literals are well-defined!")
         positive_preconditions = set()
         negative_preconditions = set()
         add_effects = set()
@@ -70,21 +102,19 @@ class PlanConverter:
             add_effects.update([p.untyped_representation for p in op.grounded_add_effects])
             delete_effects.update([p.untyped_representation for p in op.grounded_delete_effects])
 
-        next_action_pos_preconditions = [p.untyped_representation for p in next_action.grounded_positive_preconditions]
-        next_action_neg_preconditions = [p.untyped_representation for p in next_action.grounded_negative_preconditions]
+        next_action_pos_preconditions = set(
+            [p.untyped_representation for p in next_action.grounded_positive_preconditions])
+        next_action_neg_preconditions = set(
+            [p.untyped_representation for p in next_action.grounded_negative_preconditions])
         next_action_add_effects = [p.untyped_representation for p in next_action.grounded_add_effects]
         next_action_del_effects = [p.untyped_representation for p in next_action.grounded_delete_effects]
 
-        if len(negative_preconditions.intersection(next_action_pos_preconditions)) > 0 or \
-                len(positive_preconditions.intersection(next_action_neg_preconditions)) > 0 or \
-                len(add_effects.intersection(next_action_del_effects)) > 0 or \
-                len(delete_effects.intersection(next_action_add_effects)) > 0:
-            self.logger.debug("The new action combined with the current joint action is not well defined!")
-            return False
+        return self._validate_two_sided_relation(
+            positive_preconditions, negative_preconditions, add_effects, delete_effects, next_action_pos_preconditions,
+            next_action_neg_preconditions, next_action_add_effects, next_action_del_effects)
 
-        return True
-
-    def _validate_well_defined_joint_action(self, combined_actions: List[ActionCall], next_action: ActionCall,
+    def _validate_well_defined_joint_action(self, current_state: State,
+                                            combined_actions: List[ActionCall], next_action: ActionCall,
                                             next_executing_agent: str, agent_names: List[str],
                                             should_validate_concurrency_constraint: bool = True) -> bool:
         """Validates if the joint action is well-defined.
@@ -121,10 +151,12 @@ class PlanConverter:
 
         next_action_op = Operator(self.ma_domain.actions[next_action.name], self.ma_domain, next_action.parameters)
         next_action_op.ground()
+        if not next_action_op.is_applicable(current_state):
+            return False
 
         return self._validate_well_defined_action_literals(combined_actions, next_action_op)
 
-    def _create_joint_actions(self, plan_actions: List[Tuple[ActionCall, str]],
+    def _create_joint_actions(self, problem: Problem, plan_actions: List[Tuple[ActionCall, str]],
                               agent_names: List[str],
                               should_validate_concurrency_constraint: bool = True) -> List[JointActionCall]:
         """Creates the joint actions from the single agent actions.
@@ -136,6 +168,7 @@ class PlanConverter:
         """
         self.logger.info("Creating the joint actions from the single agent action plan!")
         joint_actions = []
+        current_state = create_initial_state(problem)
         while len(plan_actions) > 0:
             self.logger.debug("Initializing joint action to have only NOP for all the agents")
             joint_action = [ActionCall(NOP_ACTION, []) for _ in agent_names]
@@ -147,18 +180,22 @@ class PlanConverter:
                 break
 
             next_action, next_executing_agent = plan_actions[0]
-            while self._validate_well_defined_joint_action(joint_action, next_action, next_executing_agent,
+            while self._validate_well_defined_joint_action(current_state, joint_action, next_action,
+                                                           next_executing_agent,
                                                            agent_names, should_validate_concurrency_constraint):
                 joint_action[agent_names.index(next_executing_agent)] = plan_actions.pop(0)[0]
 
+            current_state = apply_actions(self.ma_domain, current_state, [action for action in joint_action
+                                                                          if action.name != NOP_ACTION])
             joint_actions.append(JointActionCall(joint_action))
 
         return joint_actions
 
-    def convert_plan(self, plan_file_path: Path, agent_names: List[str],
+    def convert_plan(self, problem: Problem, plan_file_path: Path, agent_names: List[str],
                      should_validate_concurrency_constraint: bool = True) -> List[JointActionCall]:
         """Converts a single agent plan to a multi-agent plan containing joint actions.
 
+        :param problem: the problem that the plan solves.
         :param plan_file_path: the path to the original single agent plan.
         :param agent_names: the names of the agents that appear in the plan.
         :param should_validate_concurrency_constraint: whether to validate the concurrency constraint.
@@ -166,7 +203,8 @@ class PlanConverter:
         """
         with open(plan_file_path, "rt") as plan_file:
             plan_data = self._extract_plan_actions(plan_file.read(), agent_names)
-            plan_actions = self._create_joint_actions(plan_data, agent_names, should_validate_concurrency_constraint)
+            plan_actions = self._create_joint_actions(problem, plan_data, agent_names,
+                                                      should_validate_concurrency_constraint)
 
         return plan_actions
 
